@@ -1,6 +1,6 @@
 import logging
 from . import util
-from . import SerialInterface, BROADCAST_NUM
+from . import SerialInterface, TCPInterface, BROADCAST_NUM
 from pubsub import pub
 import time
 import sys
@@ -17,15 +17,21 @@ testsRunning = False
 
 testNumber = 0
 
+sendingInterface = None
+
 
 def onReceive(packet, interface):
     """Callback invoked when a packet arrives"""
-    print(f"From {interface.devPath}: {packet}")
-    p = DotMap(packet)
+    if sendingInterface == interface:
+        pass
+        # print("Ignoring sending interface")
+    else:
+        # print(f"From {interface.stream.port}: {packet}")
+        p = DotMap(packet)
 
-    if p.decoded.data.typ == "CLEAR_TEXT":
-        # We only care a about clear text packets
-        receivedPackets.append(p)
+        if p.decoded.portnum == "TEXT_MESSAGE_APP":
+            # We only care a about clear text packets
+            receivedPackets.append(p)
 
 
 def onNode(node):
@@ -39,7 +45,7 @@ def subscribe():
     pub.subscribe(onNode, "meshtastic.node")
 
 
-def testSend(fromInterface, toInterface, isBroadcast=False, asBinary=False):
+def testSend(fromInterface, toInterface, isBroadcast=False, asBinary=False, wantAck=False):
     """
     Sends one test packet between two nodes and then returns success or failure
 
@@ -59,54 +65,66 @@ def testSend(fromInterface, toInterface, isBroadcast=False, asBinary=False):
     else:
         toNode = toInterface.myInfo.my_node_num
 
-    logging.info(f"Sending test packet from {fromNode} to {toNode}")
-    wantAck = True
+    logging.debug(
+        f"Sending test wantAck={wantAck} packet from {fromNode} to {toNode}")
+    global sendingInterface
+    sendingInterface = fromInterface
     if not asBinary:
         fromInterface.sendText(f"Test {testNumber}", toNode, wantAck=wantAck)
     else:
         fromInterface.sendData((f"Binary {testNumber}").encode(
             "utf-8"), toNode, wantAck=wantAck)
-    time.sleep(45)
-    return (len(receivedPackets) >= 1)
+    for sec in range(60):  # max of 60 secs before we timeout
+        time.sleep(1)
+        if (len(receivedPackets) >= 1):
+            return True
+    return False  # Failed to send
 
 
-def testThread(numTests=50):
-    logging.info("Found devices, starting tests...")
+def runTests(numTests=50, wantAck=False, maxFailures=0):
+    logging.info(f"Running {numTests} tests with wantAck={wantAck}")
     numFail = 0
     numSuccess = 0
     for i in range(numTests):
         global testNumber
         testNumber = testNumber + 1
         isBroadcast = True
+        # asBinary=(i % 2 == 0)
         success = testSend(
-            interfaces[0], interfaces[1], isBroadcast, asBinary=(i % 2 == 0))
+            interfaces[0], interfaces[1], isBroadcast, asBinary=False, wantAck=wantAck)
         if not success:
             numFail = numFail + 1
             logging.error(
                 f"Test failed, expected packet not received ({numFail} failures so far)")
         else:
             numSuccess = numSuccess + 1
-            logging.info(f"Test succeeded ({numSuccess} successes so far)")
+            logging.info(
+                f"Test succeeded {numSuccess} successes {numFail} failures so far")
 
-        if numFail >= 3:
-            for i in interfaces:
-                i.close()
-            return
+        # if numFail >= 3:
+        #    for i in interfaces:
+        #        i.close()
+        #    return
 
         time.sleep(1)
 
+    if numFail > maxFailures:
+        logging.error("Too many failures! Test failed!")
+
+    return numFail
+
+
+def testThread(numTests=50):
+    logging.info("Found devices, starting tests...")
+    runTests(numTests, wantAck=True)
+    # Allow a few dropped packets
+    runTests(numTests, wantAck=False, maxFailures=5)
+
 
 def onConnection(topic=pub.AUTO_TOPIC):
     """Callback invoked when we connect/disconnect from a radio"""
     print(f"Connection changed: {topic.getName()}")
 
-    global testsRunning
-    global interfaces
-    if (all(iface.isConnected for iface in interfaces) and not testsRunning):
-        testsRunning = True
-        t = threading.Thread(target=testThread, args=())
-        t.start()
-
 
 def openDebugLog(portName):
     debugname = "log" + portName.replace("/", "_")
@@ -129,8 +147,30 @@ def testAll():
     pub.subscribe(onReceive, "meshtastic.receive")
     global interfaces
     interfaces = list(map(lambda port: SerialInterface(
-        port, debugOut=openDebugLog(port), connectNow=False), ports))
-    for i in interfaces:
-        i.connect()
+        port, debugOut=openDebugLog(port), connectNow=True), ports))
 
-    logging.info("Ports opened, waiting for device to complete connection")
+    logging.info("Ports opened, starting test")
+    testThread()
+
+    for i in interfaces:
+        i.close()
+
+
+def testSimulator():
+    """
+    Assume that someone has launched meshtastic-native as a simulated node.
+    Talk to that node over TCP, do some operations and if they are successful
+    exit the process with a success code, else exit with a non zero exit code.
+
+    Run with
+    python3 -c 'from meshtastic.test import testSimulator; testSimulator()'
+    """
+    logging.basicConfig(level=logging.DEBUG if False else logging.INFO)
+    logging.info("Connecting to simulator on localhost!")
+    iface = TCPInterface("localhost")
+    iface.showInfo()
+    iface.localNode.showInfo()
+    iface.localNode.exitSimulator()
+    iface.close()
+    logging.info("Integration test successful!")
+    sys.exit(0)
